;
;   Modbus crc 16 bit calculation for AVR family.
;   Copyright (C) 2010  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
;
;


;*****************************************************************************
; Table driver CRC calculation has been removed to save the space required
; by the table. The CRC functions is held as short as posibe.
; 


;*****************************************************************************



.CSEG


ModbusCrc16:
;Calcola il CRC a 16 bit di una stringa assegnata utilizzando una procedura
;specifica per il protocollo Modbus. Onestamente non capisco qusto metodo di
;calcolo ma lo riporto fedelmente perche' sembra essere l'unico in grado di
;funzionare.
;Questa funzione non richiede che al messaggio siano aggiunti zeri prima
;di calcolare il CRC durante l'invio della risposta.
;Per chiamare:	X punta alla stringa di cui calcolare il CRC
;		r18 contiene la dimensione della stringa in byte
;In uscita:	r17:r16 contiene il CRC calcolato
;		X viene perso
;		r18 viene perso
		push r19		;salva i registri utilizzati
		push r20
		push ZH
		push ZL
		ser r16			;inizializza r17:r16 a 0xffff
		ser r17
		ldi ZH,high(0xa001);	;in Z il polinomio generatore
		ldi ZL,low(0xa001);
CrcLoop:	ldi r20,8		;carica il contatore dei bit
		ld r19,X+		;in r19 un byte dalla stringa
		eor r16,r19		;xor tra il data byte e CRC-L
CrcBitLoop:	clc			;azzera il carry per shiftare
		ror r17			;shifta r17:r16 a destra
		ror r16
		brcc NoCrcXor		;se C=0 salta
		eor r17,ZH		;esegue lo xor con il polinomio gen.
		eor r16,ZL
NoCrcXor:	dec r20			;decrementa il contatore dei bit
		brne CrcBitLoop		;se non zero salta
		dec r18			;decrementa il contatore dei char
		brne CrcLoop		;se non zero salta
		pop ZL			;recupera i registri
		pop ZH
		pop r20
		pop r19
		ret			;termina


